function [Source,Contribution,W] = Deflation(y,Param,FreqCycl)

% [Source,Contribution] = Deflation(x,Param,FreqCycl)
%
% Blind source separation by a deflation based method
% This function works for :
% - instantaneous or convolutive mixtures of sources
% - stationary or cyclostationary sources
% Run DemoNog to see usage examples and GenerateParamStruct to assist you
% generating a correct Param stucture array.
%
% Inputs:
% -------
% x = (Nx x Nsample) matrix containing the observations resulting from the
%   mixture (Nx sensors and Nsample samples)
% Param = structure array variables which controls the behaviour and the
%   different options of the function. Use the function GenerateParamStuct 
%   to assist you in setting the structure variable Param and to obtain 
%   more information.
% FreqCycl = (optionnal, for cyclo-stationnary sources only) array 
%   containing the cyclic frequencies >0. If you don't know what to do, we
%   recommend you not to pass a third argument. This input argument is used
%   only if Param.method = 'cyclostat'
%
% Outputs:
% -------
% Source = (Ns x Nsample) array containing the Ns separated signals. 
%   In case of i.i.d. sources, the Ns separated signals correspond to the
%   sources (up to permutation, scaling factor and delay); in the case of
%   non i.i.d. sources, the separated signals correspond to the estimation
%   of filtered versions of the Ns sources.
% Contribution = (Ns x Nx x Nsample') array containing the contribution 
%   of each source signal on each sensor. (Nsample'<=Nsample depending
%   whether the signals have been truncated during the separation process)
% W = MIMO-FIR separator coefficient (3D array, which should be used as an
%   input argument for the function MIMOfilt)
%
% -------
% Author: Pierre Jallon / Marc Castella
% Created: 22/04/2005
% Last modified: 05/07/2006
% Contact: marc.castella@int-evry.fr


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters test and initialization :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath(sprintf('%s/CycloStat/',pwd));
addpath(sprintf('%s/CycloStat/SourceExtraction/',pwd));
addpath(sprintf('%s/Stationary/KurtosisContrast/',pwd));
addpath(sprintf('%s/Stationary/QuadraticContrast/',pwd));

if (nargin==0)
    disp('You need to pass the mixture as a parameter');
    exit;    
end

if (nargin==1)
    Param = GenerateParameters;
    FreqCycl = [];
end

if (nargin==2)
    FreqCycl = [];
end


if (size(y,1)>size(y,2))
    y = y.';    
end

StillSource = true;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Deflation procedure :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

MixturePower = sum(sum(abs(y).^2));
iDetectedSource = 1;

% Variables for gradient algorithm
Direction = 0;
OldGradNorm = 1;

D = Param.LFilter; Nobserv = size(y,1);

y_trunc = y;
Ns_trunc = length(y);

Contribution = zeros(1,Nobserv,Ns_trunc);

while(StillSource)    
    
    fprintf('Extracting source %d...\n',iDetectedSource);
	switch Param.Method,
	case 'cyclostat',
	    g = GradientCyclo(y_trunc,D,Param.NbIterationMax,Param.Seuil,FreqCycl);
        W(iDetectedSource,:,:) = reshape(g.',1,Nobserv,D);
	case 'kurtosisRealValued', 
        %%% ne marche que pour des sources reelles, similaire a 'kurtosis'
        choix_mu = 'adapt'; 
		f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
		[W(iDetectedSource,:,:),crit1] = GradientKurtR(Param.Seuil,Param.NbIterationMax,y_trunc,f_ini,choix_mu);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
   	case 'kurtosis',
        choix_mu = 'adapt'; 
		f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
		[W(iDetectedSource,:,:),crit1] = GradientKurtC(Param.Seuil,Param.NbIterationMax,y_trunc,f_ini,choix_mu);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case 'quadratic', %faire evt un case 'reference' pour le cas semi-blind
        L = Param.Quadratic.L;
        Y = [];
        for d = 1:D,
            Y = [ y_trunc(:,d:end+d-D); Y];
        end;
        w = zeros(1,Nobserv*D); w(1) = 1; %w = randn(1,Nobserv*D);
        z = w*Y;
        for iterptfix = 1:Param.Quadratic.Nifix,
            [R,C] = makeRC(y_trunc,z,D);
            w = reswCw(C,R,(iDetectedSource-1)*(D+L-1),min(Nobserv*D,Param.NsourcesEffectiv*(D+L-1)));
            z = w*Y;
        end;
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
	end;

    if strcmp(Param.Method,'cyclostat'), 
    % pour raisons compatibilite avec Demog, eviter de tronquer qd on fait
    % appel aux fonctions made in Jallon
        fprintf('Cancelling its contribution ...\n');
        r = ComputeSignal(g,y_trunc);
        Source(iDetectedSource,1:length(r)) = r;
        %%%PAS TRONCATURE%%% Ns_trunc = Ns_trunc-D+1;
        % Cancelling the extracted source contribution :
        [ContributionEst,FiltreExtTemp] = SubtractSource(y_trunc,r,Param.LSubtractFiltreAC,Param.LSubtractFiltreC);
        % Save Extracted Source:
        %%%PAS TRONCATURE%%% Contribution = Contribution(:,:,1:Ns_trunc);
        Contribution(iDetectedSource,:,:) = permute(ContributionEst,[3 1 2]);
        % Updating mixture
        y_trunc = y_trunc-ContributionEst;
    else
        fprintf('Cancelling its contribution ...\n');
        r = ComputeSignal(g,y_trunc);
        Source(iDetectedSource,1:length(r)) = r;
        Ns_trunc = Ns_trunc-D+1; %%%TRONCATURE%%%
        % Cancelling the extracted source contribution :
        [ContributionEst,FiltreExtTemp] = SubtractSource(y_trunc(:,1:Ns_trunc),r(D:end),Param.LSubtractFiltreAC,Param.LSubtractFiltreC);
        % Save Extracted Source:
        Contribution = Contribution(:,:,1:Ns_trunc); %%%TRONCATURE%%%
        Contribution(iDetectedSource,:,:) = permute(ContributionEst,[3 1 2]);
        % Updating mixture
        y_trunc = y_trunc(:,1:Ns_trunc)-ContributionEst;
    end;
    
    % Still source ?
    NewMixturePower = sum(sum(abs(y_trunc).^2));
        
    if (NewMixturePower/MixturePower<Param.SeuilStopAlgo) 
        StillSource = false;
    elseif (iDetectedSource==(Param.Nsources))
        StillSource = false;
    elseif (iDetectedSource == size(y_trunc,1)),
        StillSource = false;
%%% je ne suis pas tout a fait d'avis de faire ce qu'il y a ci-dessous qui
%%% consiste a recuperer les "miettes" sur les capteurs une fois que l'on
%%% arrive a la source numero Ncapteur-1 Raison: 1) pourquoi garder la
%%% miette y(1,:) et pas y(2,:) ou ...?
%%%         2) s'il y a plus de sources que de capteurs (et qu'elle sont
%%%         faibles) on peut éventuellement les voir comme du bruit -> ça
%%%         vaut le coup d'optimiser le contraste 3) si source iid, il vaut
%%%         mieux optimiser le contraste plutot que de garder une source
%%%         filtree
%%% -> A VOIR AVEC PIERRE
    else
        iDetectedSource = iDetectedSource+1;
    end
end

% identification filtre MIMO separateur
if ~strcmp(Param.Method,'cyclostat'),
    Y = [];
    for d = 1:D,
        Y = [ y(:,d:end+d-D); Y];
    end;
    Y = Y(:,1:Ns_trunc);
    Wtmp = Source(:,1:Ns_trunc)/Y;
    W=reshape(Wtmp,size(Source,1),Nobserv,D);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Cleaning : 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rmpath(sprintf('%s/CycloStat/',pwd));
rmpath(sprintf('%s/CycloStat/SourceExtraction/',pwd));
rmpath(sprintf('%s/Stationary/KurtosisContrast/',pwd));
rmpath(sprintf('%s/Stationary/QuadraticContrast/',pwd));