function [Parameters] = GenerateParamStruct()

% [Param] = GenerateParamStruct()
%
% GenerateParamStruct assists you in creating a parameter structure array
% for the function Deflation
%
% Output:
% -------
% Param = parameter structure array
%
% Description of the parameter structure array:
% ---------------------------------------------
%   Param.Nsources = number of sources to extract
%   Param.SeuilStopAlgo = deflation stopping criterion (based on the
%       remaing power on the sensors)
%
%   Param.LFilter = length of the separating filter
%   Param.LSubtractFiltreAC = length of the anti-causal part of the
%       subtracting deflation filter
%   Param.LSubtractFiltreC = length of the causal part of the subtracting
%       deflation filter
%
%   Param.Method = method used for the extraction of one source. Possible
%       choices are:
%           1- 'kurtosisRealValued'
%           2- 'kurtosis'
%           3- 'quadratic' (see ref. [1] for a description)
%           4- 'cyclostat'
%       Choices 1,2,3 are for stationary sources only, 4 is for
%       cyclo-stationary or stationary sources.
%
%   Param.NbIterationMax = maximum number of gradient iterations (not
%       required by the 'quadratic' method)
%   Param.Seuil = gradient iteration stopping criterion (not required by
%   the 'quadratic' method)
%   
%   Param.NourcesEffectiv = number of sources which effectively contribute
%       to the mixture (assumed to be known and required by the 'quadratic'
%       method)
%   Param.Quadratic.L = length of the mixing filter (assumed to be known
%       and required by the 'quadratic' method)
%   Param.Quadratic.Nifix = number of fixed point like iterations (required
%       by the 'quadratic' method, see [1] for more details)
%
% References:
% -----------
% [1] M. Castella, S. Rhioui, E. Morean and J.-C. Pesquet, Quadratic
% Higher-Order Criteria for Iterative Blind Separation of a MIMO
% Convolutive Mixture of Sources. Accepted for publication in IEEE Trans.
% Signal Processing. 
%
% -------
% Author : Pierre Jallon / Marc Castella
% Created: 22/04/2005
% Last modified: 07/07/2006
% Contact: marc.castella@int-evry.fr

fprintf('\nNumber of sources to be extracted\n');
fprintf('---------------------------------\n');
% Maximum number of sources to be extracted
fprintf('What is the maximum number of sources you want to extract?\n');
fprintf('(If you don''t know, provide an upper-bound: <5 seems reasonnable to try first)\n');
answer = input('    Please enter a value [dflt: 3]');
disp('--');
if length(answer)==0,
    Parameters.Nsources = 3;
else
    Parameters.Nsources = str2double(answer);
    if isnan(Parameters.Nsources),
        Parameters.Nsources = 3;
        warning('Incorrect answer. Parameter set to default value');
    end;
end;
%%%%%
% Algorithm end
fprintf('After a deflation step, what is the value of the ratio:\n');
fprintf('\t (remaining power on the sensors)/(initial power on the sensors)\n');
fprintf('under which you consider that there is no more source? \n');
%fprintf('For a ratio below the given value, the deflation procedure will stop.\n');
fprintf('(The deflation procedure stops as soon as the the ration falls under the given\n');
fprintf('value. Leave blank if you don''t know or set 0 if you want to extract the above\n');
fprintf('maximum number of sources)\n');
answer = input('    Please enter a value [dflt: 1e-2]','s');
disp('--');
if (length(answer)==0)
    Parameters.SeuilStopAlgo = 1e-2;
else
    Parameters.SeuilStopAlgo = str2double(answer);
    if (isnan(Parameters.SeuilStopAlgo))
        Parameters.SeuilStopAlgo = 1e-2;
        warning('Incorrect answer. Parameter set to default value');
    end
end


fprintf('\nLengths of the filters\n');
fprintf('----------------------\n');
%%%%%
% Extracting filter length
fprintf('What is the length of the extracting filter?\n');
fprintf('(1 for instantaneous mixture, >1 for convolutive mixture)\n');
answer = input('    Please enter a value : [dflt: 1]');
disp('--');
if (length(answer)==0)
    Parameters.LFilter = 1;
else
    Parameters.LFilter = double(answer);
end

if Parameters.LFilter==1,
    fprintf('Instantaneous mixture => no causal nor anticausal part for the subtracting filter\n--\n');
    Parameters.LSubtractFiltreAC = 0;
    Parameters.LSubtractFiltreC =  0;
else,
    %%%%%
    % LSubstractFiltreAC
    answer = input('\nAnti-causal part of subtracting filter length : \nPlease enter a value : [dflt: 80]');
    disp('--');
    if (length(answer)==0)
        Parameters.LSubtractFiltreAC = 80;
    else
        Parameters.LSubtractFiltreAC = double(answer);
    end
    
    %%%%%
    % LSubstractFiltreC
    answer = input('\nCausal part of subtracting filter length : \nPlease enter a value : [dflt: 20]');
    disp('--');
    if (length(answer)==0)
        Parameters.LSubtractFiltreC = 20;
    else
        Parameters.LSubtractFiltreC = double(answer);
    end
end;



fprintf('\nChoice of the extraction method\n');
fprintf('-------------------------------\n');
fprintf('Available methods:\n');
fprintf('\t1)kurtosis, real valued signals only\n\t2)kurtosis\n\t3)quadratic contrast\n\t4)kurtosis for cyclostationary sources\n');
answer = input('    Please enter a value [dflt: 3]:');
disp('--');
if length(answer)==0,
    answer = 3;
end;
choix_methode = strvcat('kurtosisRealValued','kurtosis','quadratic','cyclostat');
   
Parameters.Method = deblank(choix_methode(answer,:));
if answer==3,
    fprintf('Parameters specific to the chosen method:\n');
    fprintf('-----------------------------------------\n');    
    
    fprintf('What is the number of sources effectively contributing to the mixture (assumed to be known)\n');
    answer = input('    Please enter a value [dflt: 3]');
    disp('--');
    if length(answer)==0,
        Parameters.NsourcesEffectiv = 3,
    else
        Parameters.NsourcesEffectiv = str2double(answer);
        if (isnan(Parameters.NsourcesEffectiv))
            Parameters.NsourcesEffectiv = 3;
            warning('Incorrect answer. Parameter set to default value');
        end;
    end;

    fprintf('What is the length of the mixing filter (assumed to be known)?\n');
    answer = input(' Please enter a value [dflt:5]');
    disp('--');
    if length(answer)==0,
        Parameters.Quadratic.L = 5;
    else
        Parameters.Quadratic.L = str2double(answer);
        if isnan(Parameters.Quadratic.L),
            Parameters.Quadratic.L = 5;
            warning('Incorrect answer. Parameter set to default value');
        end;
    end;
    
    fprintf('Number of fixed point like iterations:\n');
    answer = input('    Please enter a value [dflt: 5]');
    if length(answer)==0,
        Parameters.Quadratic.Nifix = 5;
    else
        Parameters.Quadratic.Nifix = str2double(answer);
        if isnan(Parameters.Quadratic.Nifix),
            Parameters.Quadratic.Nifix = 5;
            warning('Incorrect answer. Parameter set to default value');
        end;
    end;
else
    fprintf('Parameters specific to the chosen method:\n');    
    fprintf('-----------------------------------------\n');    
    %%%%%
    % Maximum number of iteration
    fprintf('Maximum number of gradient iterations: \n');
    answer = input('    Please enter a value : [dflt: 1000]','s');
    disp('--');
    if (length(answer)==0)
        Parameters.NbIterationMax = 1000;
    else
        Parameters.NbIterationMax = str2double(answer);
        if (isnan(Parameters.NbIterationMax))
            Parameters.NbIterationMax = 1000;
            warning('Incorrect answer. Parameter set to default value');
        end
    end
    %%%%%
    % Stop criteria
    fprintf('Gradient stop criterion : \nMinimum difference between the norm of one computed filter and the following one\n');
    answer = input('    Please enter a value [dflt: 1e-3]','s');
    disp('--');
    if (length(answer)==0)
        Parameters.Seuil = 1e-3;
    else
        Parameters.Seuil = str2double(answer);
        if (isnan(Parameters.Seuil))
            Parameters.Seuil = 1e-3;
            warning('Incorrect answer. Parameter set to default value');
        end
    end
end;

disp('------------------------------');
disp('Generated parameter structure:'); disp(Parameters);
disp('------------------------------');

