function [MSE,newx,newy,Permutx] = elimamb_g3(x,y,decalmax)

% [MSE,newx,newy,Permutx] = elimamb_g3(x,y,decalmax)
%
% eliminate ambiguities of convolutive blind source separation by a 
% correlation method
% elimamb_g3 eliminates the following ambiguities:
%   - permutation
%   - time delays
%   - sign ambiguities / (or phase indetermination for complex signals)
%
% Inputs:
% -------
% x = (mx x Lx) matrix containing mx signals of each Lx samples
% y = (my x Ly) matrix containing my signals of each Ly samples
% decalmax = maximum delay ambiguity between x and y
%
% Outputs: 
% -------
% MSE = mean square error between x and y
% newx = truncated and permuted version of x
% newy = truncated and permuted version of y
% Permutx = permutation matrix. If my<=mx and Ly<=Lx then the rows of y are 
%   are not permuted to obtain newy. The rows of x are permuted by Permutx
%   to obtain newx
%
% -------
% Author: Jean-Christophe Pesquet / Marc Castella
% Created: ??
% Last modified: 03/07/2006
% Contact: marc.castella@int-evry.fr

% Autres rq:
% supprime entre les signaux x et y une ambiguite de type 
% permutation, changement de signe et decalage temporel
% par une methode de correlation
% 
% x = matrice mx x Lx
% y = matrice my x Ly avec Lx >= Ly (sinon, x et y inverses!)
% changement par rapport elimamb_g: my < mx autorise
% donc: my <= mx et Ly<= Lx
% ----------             ---------
% decalmax = decalage temporel maximal
%
% MSE  = erreur quadratique moyenne entre x et y
% newx = version tronquee de x, eventuellement permutee
% newy = y, recale par rapport a newx
% Les lignes de newy ne sont pas changees par rapport a celles de y!!
% Marc Castella 27/11/2002
% Pour retrouver le meme ordre des lignes dans x, 
% faire newx <- Permutx*x et les lignes de newx correspondent a celles newy
% (ou y c'est pareil)
% MC 23/05/2003, bug sur Permutx corrige le 14/08/2004.

if size(x,2)<size(y,2),
    toto = x;
    x = y;
    y = toto;
end;
Ly = size(y,2); Lx = size(x,2);

mx = size(x,1); my=size(y,1);
if mx < my,
    error('mx < my'); %%%error a la place de disp %%%% %break;
end;

% calculate maximum cross-correlations (value rmax and position dmax)
% for each pair of sources
for ii=1:mx
    for jj=1:my 
        r = xcorr(x(ii,:),y(jj,:),decalmax);
        %r = [ zeros(1,decalmax) r(decalmax+1:2*decalmax+1) ];
        [rmax(ii,jj),dmax(ii,jj)] = max(abs(r));
        rmax(ii,jj) = r(dmax(ii,jj));
    end;
end;

% find maximally correlated sources and corresponding delay
xtemp = zeros(my,Lx);
ytemp = zeros(size(y));
dmaxtemp = zeros(1,my);
rmaxtemp = zeros(1,my);
Permuty = zeros(my);
Permutx = zeros(my,mx);
for ii = 1:my
    [iic,jjc] = find(abs(rmax)==max(abs(rmax(:))));
    iic = iic(1); % non obligatoire, resoud le pb si le max trouve non unique
    jjc = jjc(1); % idem
    %[iic jjc]
    ytemp(ii,:) = y(jjc,:); xtemp(ii,:) = x(iic,:); 
    Permuty(jjc,ii) = 1; %ligne jjc de y passe en ii        
    Permutx(jjc,iic) = 1; %%%ligne iic de x passe en ii
    dmaxtemp(ii) = dmax(iic,jjc);
    rmaxtemp(ii) = rmax(iic,jjc);
    rmax(iic,:) = zeros(1,my);
    rmax(:,jjc) = zeros(mx,1);
end
dmax = dmaxtemp;
y = ytemp; x = xtemp;
rmax = rmaxtemp;

% make signals of equal length
dmax = dmax - (decalmax+1);
deb_newx = max(max(dmax)+1,1);
fin_newx = min(Ly+min(dmax),Lx);
newx = x(:,deb_newx:fin_newx);
newy = zeros(size(newx));
Lnew = size(newx,2);
for ii = 1:my
%     if dmax(ii)==0,
%         newy(ii,:) = y(ii,deb_newx:Lnew-1+deb_newx);
    %if dmax(ii)>=0,
        newy(ii,:) = y(ii,deb_newx-dmax(ii):Lnew-1+deb_newx-dmax(ii));
    %elseif dmax(ii)<0,
        %newy(ii,:) = y(ii,deb_newx-dmax(ii):Lnew-1+deb_newx-dmax(ii));
        %end;
end

% determine sign
alpha = diag(sign(rmax)); %pour x complexe, sign(x) = x./abs(x);
newy = alpha*newy;

% permutation
newx = Permuty*newx;
newy = Permuty*newy;

% calculate error
MSE = mean((abs(newx'-newy')).^2);
%MSE = (newx-newy)*(newx-newy)'/Lnew

